//
//  KBViewController.m
//  ------------------
//
//  Keith Blount 2005
//

/*
 *	NOTE
 *	----
 *	A very annoying bug in AppKit means that you cannot bind anything directly to File's Owner - doing
 *	so causes the bound object to retain File's Owner, meaning that File's Owner never gets released.
 *	KBViewController is designed to act as File's Owner - it is essentially an NSWindowController
 *	for views. This means that if you bind anything directly to it, there will be a retain cycle and it
 *	will never get released. To work around this until Apple provide a solution, never bind anything
 *	directly to File's Owner. Instead, create an NSObjectController and bind to that. Then call -setContent:
 *	in code and bind directly to the model, *not* to an instance of this class. In the case of an
 *	NSArrayController, you may need to call -setContent: whenever the array changes, for instance.
 *	Slightly annoying, but this does fix the retain/release bug.
 */

#import "KBViewController.h"


@implementation KBViewController

- (id)init
{
	return [self initWithViewNibName:nil];
}

// Designated initialiser
- (id)initWithViewNibName:(NSString *)nibName;
{
	if (self = [super init])
	{
		if (!nibName)
		{
			[self release];
			return nil;
		}
		
		// By using NSNib, we can get a reference to all of the top-level objects within
		// the nib when unarchiving it. This allows us to release all of those objects
		// (which is our responsibility) on dealloc without having to create an outlet
		// for all of them.
		
		NSNib *nib = [[[NSNib alloc] initWithNibNamed:nibName bundle:nil] autorelease];
		
		NSArray *topItems;
		if (![nib instantiateNibWithOwner:self topLevelObjects:&topItems])
		{
			[self release];
			return nil;
		}
		topLevelNibObjects = [topItems copy];
		
		viewNibName = [nibName retain];		// Save the nib name
	}
	return self;
}

- (void)dealloc
{	
	[viewNibName release];
	
	// Release all of the top-level objects in the nib file
	[topLevelNibObjects makeObjectsPerformSelector:@selector(release)];
	[topLevelNibObjects release];
	[super dealloc];
}

- (NSView *)mainView
{
	return mainView;
}

- (NSWindow *)window
{
	return [mainView window];
}

- (NSString *)viewNibName
{
	return viewNibName;
}

@end
